/**
 * OpenGL RedBook Shader.
 * Examples 7.8, 7.9 and 7.10.
 *
 * Reflections are happening in camera space
 */
#version 330 core

in Vertex {
    vec3 Position;
    vec3 Normal;
    vec3 Colour;
    vec2 TexPos;
};

out vec4 FragColour;

const float constant = 1.0;
const float linear = 0.022;
const float quadratic = 0.0019;

float specPower = 0.5;
const float shininess = 8;

uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;

uniform mat4 MVMatrix;
uniform vec3 viewerPos_ws;

struct DirectionalLight {
    vec3 direction;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

struct Material {
    vec3 emission;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

uniform DirectionalLight light;
uniform Material material;

const int hasPos = 0;

vec4 calcDirLight(DirectionalLight light, vec3 Normal, vec3 viewDir, vec4 specPx, vec4 diffPx) {
    vec3 lightDir = normalize( ( vec4(light.direction, 1) * MVMatrix).xyz - Position.xyz );
    vec4 ambient = 0.1 * vec4( light.ambient, 1);

    vec3 reflectDir = reflect( -lightDir, Normal );
    float spec = pow( max( dot(viewDir, reflectDir), 0.0), shininess);
    vec4 specular = 1.0 * vec4( light.specular, 1) * specPower * (spec * specPx);

    float diff = max( dot(Normal, lightDir), 0.0 );
    vec4 diffuse = vec4(light.diffuse,1) * (diff * diffPx);

    if ( hasPos == 1 ) {
        vec3 lightPos = ( vec4(light.direction, 1) * MVMatrix).xyz;
        float distance = length( lightPos - Position.xyz );
        float att = 1.0 / (constant +
            (linear * distance) +
            (quadratic * (distance * distance)));

        ambient *= att;
        diffuse *= att;
        specular *= att;
    }

    return (ambient + diffuse + specular);
}

void main() {
    vec3 viewDir = normalize(-Position);

    vec4 diffPx = vec4(material.diffuse, 1);
    vec4 specPx = vec4(material.specular, 1);
    if ( hasPos != 1) {
        diffPx *= texture(diffuseTexture, TexPos);
        specPx *= texture(specularTexture, TexPos);
    }

    FragColour = vec4(Colour, 1);
    FragColour *= calcDirLight(light, Normal, viewDir, specPx, diffPx);
}